/*******************************************************************
 *
 *	File:		CArloConfig.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <e32std.h>
#include <f32file.h>
#include <e32svr.h>
#include <e32cons.h>

#include "CArloConfig.h"

#define KKernelString		_L("image")
#define KInitRdString		_L("initrd")
#define KBootString			_L("boot")
#define KForegroundColour	_L("fgcolour")
#define KBackgroundColour	_L("bgcolour")
#define KArchNumber			_L("arch")

#define KTimeoutString		_L("timeout")
#define KDefaultBoot		_L("default")
#define KMachineName		_L("machine")
#define KUseBoot2nd			_L("useboot2nd")


CArloConfig::~CArloConfig()
{
	delete iArloBootItems;
	iArloBootItems = NULL;
}

CArloConfig::CArloConfig() : iUseBoot2nd(ETrue)
{
}

CArloConfig* CArloConfig::ReadConfigFileLC()
{
	CArloConfig*	arloConfig = new(ELeave) CArloConfig;
	CleanupStack::PushL(arloConfig);
	arloConfig->ConstructL();
	return arloConfig;
}

void CArloConfig::NewBootItem(const TDesC& aBootItem)
{
	TArloBootItem	item;

	if (iArloBootItems->Count() < 10)
	{
		item.iKernelFileName = _L("");
		item.iInitrdFileName = _L("");
		item.iKernelLoadAdr = 0;
		item.iInitrdLoadAdr = 0;
		item.iDescription = aBootItem;
		item.iBootString = _L("");
		item.iForegroundColour = 0;
		item.iBackgroundColour = 0;
		item.iHasColoursSet = EFalse;
		item.iArchNumber = -1;

		iArloBootItems->AppendL(item);
	}
}

TBool CArloConfig::GetNumber(TLex& aLex, TUint& aNumber)
{
	TChar		c1, c2;
	TBool		success = EFalse;

	if (aLex.Remainder().Length() > 2)
	{
		c1= aLex.Get();
		c2= aLex.Get();
		if (c1 == '0' && (c2 == 'x' || c2 == 'X'))
		{
			success = aLex.Val(aNumber, EHex) == KErrNone;
		}
		else
		{
			aLex.UnGet();
			aLex.UnGet();
		}
	}

	if (!success)
	{
		success = aLex.Val(aNumber, EDecimal) == KErrNone;
	}
	return success;
}


void CArloConfig::ParseFileNameAndLoadAdr(const TDesC& aString, TDes& aFileName, TUint& aLoadAdr)
{
	TInt	pos = aString.LocateF(TChar('@'));

	if ((pos > 0) && (pos + 1 < aString.Length()))
	{
		TLex		input(aString.Left(pos));

		input.SkipSpaceAndMark();
		input.SkipCharacters();
		if (input.TokenLength() != 0)
		{
			TPtrC		token = input.MarkedToken();

			input.Assign(aString.Mid(pos+1));
			input.SkipSpaceAndMark();
			if (!GetNumber(input, aLoadAdr))
				aLoadAdr = 0;
	
			aFileName.Copy(token);
		}
	}
	else
	{
		aFileName.Copy(aString);
		aLoadAdr = 0;
	}
}

void CArloConfig::ParseItems(const TDesC& aToken, const TDesC& aValue)
{
	RDebug::Print(_L("  token = -%S-"), &aToken);
	RDebug::Print(_L("  value = -%S-"), &aValue);
	
	TInt	count = iArloBootItems->Count();
	if (aToken.CompareF(KKernelString) == 0)
	{
		if (count > 0 && count<10)
		{
			TUint	loadAdr;
			
			ParseFileNameAndLoadAdr(aValue, iArloBootItems->At(count - 1).iKernelFileName, loadAdr);
			iArloBootItems->At(count - 1).iKernelLoadAdr = loadAdr;
		}
	}
	else if (aToken.CompareF(KInitRdString) == 0)
	{
		if (count > 0 && count<10)
		{
			TUint	loadAdr;
	
			ParseFileNameAndLoadAdr(aValue, iArloBootItems->At(count - 1).iInitrdFileName, loadAdr);
			iArloBootItems->At(count - 1).iInitrdLoadAdr = loadAdr;
		}
	}
	else if (aToken.CompareF(KBootString) == 0)
	{
		if (count > 0 && count<10)
			iArloBootItems->At(count - 1).iBootString = aValue;
	}
	else if (aToken.CompareF(KTimeoutString) == 0)
	{
		TLex	input(aValue);

		TInt	timeout;
		if (input.Val(timeout) == KErrNone)
			iTimeout = timeout;
	}
	else if (aToken.CompareF(KDefaultBoot) == 0)
	{
		iDefaultBootItemString = aValue;
	}
	else if (aToken.CompareF(KMachineName) == 0)
	{
		iMachineName = aValue;
	}
	else if (aToken.CompareF(KForegroundColour) == 0)
	{
		TLex	input(aValue);

		TUint	colour;
		if (GetNumber(input, colour))
		{
			iArloBootItems->At(count - 1).iForegroundColour = colour;
			iArloBootItems->At(count - 1).iHasColoursSet = ETrue;
		}
	}
	else if (aToken.CompareF(KBackgroundColour) == 0)
	{
		TLex	input(aValue);

		TUint	colour;
		if (GetNumber(input, colour))
		{
			iArloBootItems->At(count - 1).iBackgroundColour = colour;
			iArloBootItems->At(count - 1).iHasColoursSet = ETrue;
		}
	}
	else if (aToken.CompareF(KArchNumber) == 0)
	{
		TLex	lex(aValue);

		TUint	archno;
		if (CArloConfig::GetNumber(lex, archno))
			iArloBootItems->At(count - 1).iArchNumber = (TInt) archno;
	}
	else if (aToken.CompareF(KUseBoot2nd) == 0)
	{
		if (((aValue.CompareF(_L("yes")) == 0) || (aValue.CompareF(_L("true")) == 0)))
			iUseBoot2nd = ETrue;
		if (((aValue.CompareF(_L("no")) == 0) || (aValue.CompareF(_L("false")) == 0)))
			iUseBoot2nd = EFalse;
	}
}


void CArloConfig::ParseLine(const TDesC& aLine)
{
	RDebug::Print(_L("line = %S"), &aLine);

	if (aLine.Length() != 0)
	{
		if (aLine.Left(1) != _L("#"))
		{
			if ((aLine.Left(1) == _L("[")) && aLine.Right(1) == _L("]"))
			{
				NewBootItem(aLine.Mid(1, aLine.Length() - 2));
			}
			else
			{
				TInt	pos = aLine.LocateF(TChar('='));
			
				if ((pos > 0) && (pos + 1 < aLine.Length()))
				{
					TLex		input(aLine.Left(pos));

					input.SkipSpaceAndMark();
					input.SkipCharacters();
					if (input.TokenLength() != 0)
					{
						TPtrC		token = input.MarkedToken();

						input.Assign(aLine.Mid(pos+1));
						input.SkipSpaceAndMark();
						TPtrC		value = input.Remainder();
						ParseItems(token, value);
					}
				}
			}
		}
	}
}

void CArloConfig::ReadConfigFileL(RFs& aFs, const TDesC& aFileName)
{
	RFile			file;

	User::LeaveIfError(file.Open(aFs, aFileName, EFileRead));

	TInt size;
	User::LeaveIfError(file.Size(size));

	HBufC *buf = HBufC::NewLC(size);
	TPtr	des = buf->Des();
	User::LeaveIfError(file.Read(des, size));

	TBool	finished = EFalse;
	while(!finished)
	{
		TInt pos = des.LocateF(TChar('\n'));
		if (pos >= 0)
		{
			TBool	isDosLine = (des.LocateF(TChar('\r')) >= 0);
			
			ParseLine(des.Left( isDosLine ? pos-1 : pos));
			des = des.Mid(pos + 1);
		}
		else
		{
			ParseLine(des);
			finished = ETrue;
		}
	}

	file.Close();
	CleanupStack::PopAndDestroy();		// buf

	iConfigFile = aFileName;
	if ((iTimeout < -1) || (iTimeout > 30))
	{
		iTimeout = 30;
	}
}


void CArloConfig::Defaults()
{
	iTimeout = -1;
	iDefaultBootItem = 0;
	iArloBootItems->Reset();
	iDefaultBootItemString = _L("");
	iConfigFile = _L("<no config file>");

	// set default machine name
	TMachineInfoV1Buf		machineInfo;
	TInt err = UserHal::MachineInfo(machineInfo);
	if (err == KErrNone)
	{
		iMachineName = machineInfo().iMachineName;
	}
	else
	{
		iMachineName = _L("<unknown machine>");
	}
}


void CArloConfig::SetDefaultBootItem()
{
	TInt count = iArloBootItems->Count();
	for (TInt i=0; i<count ; i++)
	{
		if (iArloBootItems->At(i).iDescription.CompareF(iDefaultBootItemString) == 0)
		{
			iDefaultBootItem = i;
			break;
		}
	}
}


void CArloConfig::ConstructL()
{
	const TText**		arloFile;
	const TText*		arloFileLocations[] =
	{
		_S("c:\\arlo.cfg"),
		_S("d:\\arlo.cfg"),
		_S("c:\\arlo\\arlo.cfg"),
		_S("d:\\arlo\\arlo.cfg"),
		NULL
	};

	iArloBootItems = new(ELeave) CArrayFixFlat<TArloBootItem>(5);

	RFs		fs;
	User::LeaveIfError(fs.Connect());
	TInt	err;
	Defaults();

	arloFile = &arloFileLocations[0];
	while(*arloFile)
	{
		TRAP(err, ReadConfigFileL(fs, TPtrC(*arloFile)));
		if (err == KErrNone)
			break;
		
		Defaults();
		arloFile++;
	}
	if (err == KErrNone)
	{
		SetDefaultBootItem();
	}

	fs.Close();
}
